@@ -53,7 +53,9 @@ enum
   GCONF_PROP_HUE,
   GCONF_PROP_VIDEO_PATH,
   GCONF_PROP_PHOTO_PATH,
-  GCONF_PROP_ENABLE_DELETE
+  GCONF_PROP_ENABLE_DELETE,
+  GCONF_PROP_BURST_DELAY,
+  GCONF_PROP_BURST_REPEAT
 };
 
 GType        cheese_gconf_get_type (void);
