@@ -39,6 +39,10 @@
 #include <gtk/gtk.h>
 #include <libebook/e-book.h>
 
+#ifdef HAVE_MMKEYS
+  #include <X11/XF86keysym.h>
+#endif /* HAVE_MMKEYS */
+
 #ifdef HILDON
   #include <hildon/hildon-program.h>
 #endif
@@ -52,7 +56,6 @@
 #include "cheese-window.h"
 #include "ephy-spinner.h"
 #include "gst-audio-play.h"
-#include "gedit-message-area.h"
 #include "cheese-no-camera.h"
 #include "cheese-prefs-dialog.h"
 #include "cheese-flash.h"
@@ -61,11 +64,14 @@
 #define FULLSCREEN_POPUP_HEIGHT    40
 #define FULLSCREEN_TIMEOUT         5 * 1000
 #define FULLSCREEN_EFFECTS_TIMEOUT 15
+#define DEFAULT_WINDOW_WIDTH       600
+#define DEFAULT_WINDOW_HEIGHT      450
 
 typedef enum
 {
   WEBCAM_MODE_PHOTO,
-  WEBCAM_MODE_VIDEO
+  WEBCAM_MODE_VIDEO,
+  WEBCAM_MODE_BURST
 } WebcamMode;
 
 typedef enum
@@ -117,15 +123,17 @@ typedef struct
   GtkWidget *countdown_frame_fullscreen;
   GtkWidget *countdown;
   GtkWidget *countdown_fullscreen;
-  GtkWidget *message_area_frame;
-  GtkWidget *message_area;
+  GtkWidget *info_bar_frame;
+  GtkWidget *info_bar;
 
   GtkWidget *button_effects;
   GtkWidget *button_photo;
   GtkWidget *button_video;
+  GtkWidget *button_burst;
   GtkWidget *button_effects_fullscreen;
   GtkWidget *button_photo_fullscreen;
   GtkWidget *button_video_fullscreen;
+  GtkWidget *button_burst_fullscreen;
   GtkWidget *button_exit_fullscreen;
 
   GtkWidget *image_take_photo;
@@ -165,6 +173,7 @@ typedef struct
   GtkActionGroup *actions_photo;
   GtkActionGroup *actions_toggle;
   GtkActionGroup *actions_video;
+  GtkActionGroup *actions_burst;
   GtkActionGroup *actions_fullscreen;
 
   GtkUIManager *ui_manager;
@@ -173,68 +182,24 @@ typedef struct
 
   int audio_play_counter;
 
+  gint repeat_count;
+
   CheeseFlash *flash;
 } CheeseWindow;
 
+static void cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_window);
+
 void
 cheese_window_bring_to_front (gpointer data)
 {
   CheeseWindow *cheese_window     = data;
-  guint32       startup_timestamp = gdk_x11_get_server_time (GTK_WIDGET (cheese_window->window)->window);
+  guint32       startup_timestamp = gdk_x11_get_server_time (gtk_widget_get_window (GTK_WIDGET (cheese_window->window)));
 
-  gdk_x11_window_set_user_time (GTK_WIDGET (cheese_window->window)->window, startup_timestamp);
+  gdk_x11_window_set_user_time (gtk_widget_get_window (GTK_WIDGET (cheese_window->window)), startup_timestamp);
 
   gtk_window_present (GTK_WINDOW (cheese_window->window));
 }
 
-/* Make url in about dialog clickable */
-static void
-cheese_about_dialog_handle_url (GtkAboutDialog *dialog, const char *url, gpointer data)
-{
-  GError    *error = NULL;
-  GtkWidget *error_dialog;
-  GdkScreen *screen;
-
-  screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
-  gtk_show_uri (screen, url, gtk_get_current_event_time (), &error);
-  if (error != NULL)
-  {
-    error_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
-                                           GTK_DIALOG_DESTROY_WITH_PARENT,
-                                           GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-                                           _("Failed to open browser to show:\n%s"), url);
-    gtk_dialog_run (GTK_DIALOG (error_dialog));
-    gtk_widget_destroy (error_dialog);
-    g_error_free (error);
-  }
-}
-
-/* Make email in about dialog clickable */
-static void
-cheese_about_dialog_handle_email (GtkAboutDialog *dialog, const char *email, gpointer data)
-{
-  char      *uri;
-  GError    *error = NULL;
-  GtkWidget *error_dialog;
-  GdkScreen *screen;
-
-  uri = g_strconcat ("mailto:", email, NULL);
-
-  screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
-  gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
-  if (error != NULL)
-  {
-    error_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
-                                           GTK_DIALOG_DESTROY_WITH_PARENT,
-                                           GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-                                           _("Failed to open email client to send message to:\n%s"), email);
-    gtk_dialog_run (GTK_DIALOG (error_dialog));
-    gtk_widget_destroy (error_dialog);
-    g_error_free (error);
-  }
-  g_free (uri);
-}
-
 static char *
 audio_play_get_filename (CheeseWindow *cheese_window)
 {
@@ -258,6 +223,36 @@ cheese_window_delete_event_cb (GtkWidget *widget, GdkEvent event, gpointer data)
   return FALSE;
 }
 
+static gboolean
+cheese_window_key_press_event_cb (GtkWidget *win, GdkEventKey *event, CheeseWindow *cheese_window)
+{
+#ifndef HAVE_MMKEYS
+  return FALSE;
+#else
+  /* If we have modifiers, and either Ctrl, Mod1 (Alt), or any
+   * of Mod3 to Mod5 (Mod2 is num-lock...) are pressed, we
+   * let Gtk+ handle the key */
+  if (event->state != 0
+      && ((event->state & GDK_CONTROL_MASK)
+          || (event->state & GDK_MOD1_MASK)
+          || (event->state & GDK_MOD3_MASK)
+          || (event->state & GDK_MOD4_MASK)
+          || (event->state & GDK_MOD5_MASK)))
+    return FALSE;
+
+  switch (event->keyval)
+  {
+    case XF86XK_WebCam:
+
+      /* do stuff */
+      cheese_window_action_button_clicked_cb (NULL, cheese_window);
+      return TRUE;
+  }
+
+  return FALSE;
+#endif /* !HAVE_MMKEYS */
+}
+
 static void
 cheese_window_fullscreen_clear_timeout (CheeseWindow *cheese_window)
 {
@@ -288,7 +283,7 @@ cheese_window_fullscreen_set_timeout (CheeseWindow *cheese_window)
   GSource *source;
 
   cheese_window_fullscreen_clear_timeout (cheese_window);
-  
+
   source = g_timeout_source_new (FULLSCREEN_TIMEOUT);
 
   g_source_set_callback (source, cheese_window_fullscreen_timeout_cb, cheese_window, NULL);
@@ -322,8 +317,8 @@ cheese_window_fullscreen_motion_notify_cb (GtkWidget      *widget,
     {
       cheese_window_fullscreen_show_bar (cheese_window);
     }
-    
-    //don't set the timeout in effect-chooser mode
+
+    /* don't set the timeout in effect-chooser mode */
     if (gtk_notebook_get_current_page (GTK_NOTEBOOK (cheese_window->notebook)) != 1)
       cheese_window_fullscreen_set_timeout (cheese_window);
   }
@@ -357,11 +352,11 @@ cheese_window_toggle_fullscreen (GtkWidget *widget, CheeseWindow *cheese_window)
                       cheese_window);
 
     gtk_window_fullscreen (GTK_WINDOW (cheese_window->window));
-    
+
     gtk_widget_set_size_request (cheese_window->effect_alignment, -1, FULLSCREEN_POPUP_HEIGHT);
     cheese_window_fullscreen_show_bar (cheese_window);
-    
-    //don't set the timeout in effect-chooser mode
+
+    /* don't set the timeout in effect-chooser mode */
     if (gtk_notebook_get_current_page (GTK_NOTEBOOK (cheese_window->notebook)) != 1)
       cheese_window_fullscreen_set_timeout (cheese_window);
 
@@ -372,7 +367,7 @@ cheese_window_toggle_fullscreen (GtkWidget *widget, CheeseWindow *cheese_window)
     gtk_widget_show_all (cheese_window->window);
     gtk_widget_hide_all (cheese_window->fullscreen_popup);
     gtk_widget_modify_bg (cheese_window->window, GTK_STATE_NORMAL, NULL);
-    
+
     g_signal_handlers_disconnect_by_func (cheese_window->window,
                                           (gpointer) cheese_window_fullscreen_motion_notify_cb, cheese_window);
     g_signal_handlers_disconnect_by_func (cheese_window->screen,
@@ -413,11 +408,11 @@ cheese_window_photo_saved_cb (CheeseWebcam *webcam, CheeseWindow *cheese_window)
   gdk_threads_leave ();
 }
 
-
 static void
 cheese_window_video_saved_cb (CheeseWebcam *webcam, CheeseWindow *cheese_window)
 {
   gdk_threads_enter ();
+
   /* TODO look at this g_free */
   g_free (cheese_window->video_filename);
   cheese_window->video_filename = NULL;
@@ -447,6 +442,7 @@ cheese_window_cmd_close (GtkWidget *widget, CheeseWindow *cheese_window)
   g_object_unref (cheese_window->actions_preferences);
   g_object_unref (cheese_window->actions_file);
   g_object_unref (cheese_window->actions_video);
+  g_object_unref (cheese_window->actions_burst);
   g_object_unref (cheese_window->actions_fullscreen);
   g_object_unref (cheese_window->gconf);
 
@@ -940,7 +936,7 @@ cheese_window_cmd_command_line (GtkAction *action, CheeseWindow *cheese_window)
 static void
 cheese_window_cmd_help_contents (GtkAction *action, CheeseWindow *cheese_window)
 {
-  GError  *error = NULL;
+  GError    *error = NULL;
   GdkScreen *screen;
 
   screen = gtk_widget_get_screen (GTK_WIDGET (cheese_window));
@@ -1157,6 +1153,10 @@ cheese_window_effect_button_pressed_cb (GtkWidget *widget, CheeseWindow *cheese_
     {
       gtk_action_group_set_sensitive (cheese_window->actions_photo, TRUE);
     }
+    else if (cheese_window->webcam_mode == WEBCAM_MODE_BURST)
+    {
+      gtk_action_group_set_sensitive (cheese_window->actions_burst, TRUE);
+    }
     else
     {
       gtk_action_group_set_sensitive (cheese_window->actions_video, TRUE);
@@ -1174,6 +1174,7 @@ cheese_window_effect_button_pressed_cb (GtkWidget *widget, CheeseWindow *cheese_
     gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture_fullscreen), FALSE);
     gtk_action_group_set_sensitive (cheese_window->actions_photo, FALSE);
     gtk_action_group_set_sensitive (cheese_window->actions_video, FALSE);
+    gtk_action_group_set_sensitive (cheese_window->actions_burst, FALSE);
   }
 }
 
@@ -1200,7 +1201,7 @@ cheese_window_countdown_picture_cb (gpointer data)
   {
     cheese_flash_fire (cheese_window->flash);
     shutter_filename = audio_play_get_filename (cheese_window);
-    audio_play = gst_audio_play_file (shutter_filename, &error);
+    audio_play       = gst_audio_play_file (shutter_filename, &error);
     if (!audio_play)
     {
       g_warning ("%s", error ? error->message : "Unknown error");
@@ -1212,7 +1213,7 @@ cheese_window_countdown_picture_cb (gpointer data)
 }
 
 static void
-cheese_window_no_camera_message_area_response (GtkWidget *widget, gint response_id, GtkWidget *cheese_window)
+cheese_window_no_camera_info_bar_response (GtkWidget *widget, gint response_id, CheeseWindow *cheese_window)
 {
   GError  *error = NULL;
   gboolean ret;
@@ -1291,6 +1292,55 @@ cheese_window_escape_key_cb (CheeseWindow *cheese_window,
   return TRUE;
 }
 
+static gboolean
+cheese_window_take_burst_photo (gpointer data)
+{
+  gboolean      countdown;
+  CheeseWindow *cheese_window = (CheeseWindow *) data;
+
+  g_object_get (cheese_window->gconf, "gconf_prop_countdown", &countdown, NULL);
+  if (countdown)
+  {
+    if (cheese_window->isFullscreen)
+    {
+      cheese_countdown_start ((CheeseCountdown *) cheese_window->countdown_fullscreen,
+                              cheese_window_countdown_picture_cb,
+                              cheese_window_countdown_hide_cb,
+                              (gpointer) cheese_window);
+      gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->fullscreen_bar), 1);
+
+      /* show bar, start timeout
+       * ATTENTION: if the countdown is longer than FULLSCREEN_TIMEOUT,
+       * the bar will disapear before the countdown ends
+       */
+      cheese_window_fullscreen_show_bar (cheese_window);
+      cheese_window_fullscreen_set_timeout (cheese_window);
+    }
+    else
+    {
+      cheese_countdown_start ((CheeseCountdown *) cheese_window->countdown,
+                              cheese_window_countdown_picture_cb,
+                              cheese_window_countdown_hide_cb,
+                              (gpointer) cheese_window);
+      gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->notebook_bar), 1);
+    }
+  }
+  else
+  {
+    cheese_window_countdown_picture_cb (cheese_window);
+  }
+
+  gtk_widget_set_sensitive (cheese_window->take_picture, FALSE);
+  gtk_widget_set_sensitive (cheese_window->take_picture_fullscreen, FALSE);
+
+  cheese_window->repeat_count--;
+  if (cheese_window->repeat_count <= 0)
+  {
+    return FALSE;
+  }
+  return TRUE;
+}
+
 static void
 cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_window)
 {
@@ -1298,44 +1348,29 @@ cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_
 
   if (cheese_window->webcam_mode == WEBCAM_MODE_PHOTO)
   {
-    gboolean countdown;
-    g_object_get (cheese_window->gconf, "gconf_prop_countdown", &countdown, NULL);
-    if (countdown)
-    {
-      if (cheese_window->isFullscreen)
-      {
-        cheese_countdown_start ((CheeseCountdown *) cheese_window->countdown_fullscreen,
-                                cheese_window_countdown_picture_cb,
-                                cheese_window_countdown_hide_cb,
-                                (gpointer) cheese_window);
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->fullscreen_bar), 1);
-
-        /* show bar, start timeout
-         * ATTENTION: if the countdown is longer than FULLSCREEN_TIMEOUT,
-         * the bar will disapear before the countdown ends
-         */
-        cheese_window_fullscreen_show_bar (cheese_window);
-        cheese_window_fullscreen_set_timeout (cheese_window);
-      }
-      else
-      {
-        cheese_countdown_start ((CheeseCountdown *) cheese_window->countdown,
-                                cheese_window_countdown_picture_cb,
-                                cheese_window_countdown_hide_cb,
-                                (gpointer) cheese_window);
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->notebook_bar), 1);
-      }
-    }
-    else
-    {
-      cheese_window_countdown_picture_cb (cheese_window);
-    }
-
-    gtk_widget_set_sensitive (cheese_window->take_picture, FALSE);
-    gtk_widget_set_sensitive (cheese_window->take_picture_fullscreen, FALSE);
+    cheese_window->repeat_count = 1;
+    cheese_window_take_burst_photo (cheese_window);
 
     /* FIXME: set menu inactive */
   }
+  else if (cheese_window->webcam_mode == WEBCAM_MODE_BURST)
+  {
+    guint    repeat_delay = 1000;
+    gboolean countdown;
+
+    g_object_get (cheese_window->gconf, "gconf_prop_burst_delay", &repeat_delay, NULL);
+    g_object_get (cheese_window->gconf, "gconf_prop_burst_repeat", &cheese_window->repeat_count, NULL);
+    g_object_get (cheese_window->gconf, "gconf_prop_countdown", &countdown, NULL);
+
+    if (countdown && repeat_delay < 5000)
+    {
+      /* A countdown takes 4 seconds, leave some headroom before repeating it.
+       * Magic number chosen via expiriment on Netbook */
+      repeat_delay = 5000;
+    }
+
+    g_timeout_add (repeat_delay, &cheese_window_take_burst_photo, cheese_window);
+  }
   else if (cheese_window->webcam_mode == WEBCAM_MODE_VIDEO)
   {
     if (!cheese_window->recording)
@@ -1406,6 +1441,7 @@ static const GtkToggleActionEntry action_entries_fullscreen[] = {
 static const GtkRadioActionEntry action_entries_toggle[] = {
   {"Photo", NULL, N_("_Photo"), NULL, NULL, 0},
   {"Video", NULL, N_("_Video"), NULL, NULL, 1},
+  {"Burst", NULL, N_("_Burst"), NULL, NULL, 2},
 };
 
 static const GtkActionEntry action_entries_file[] = {
@@ -1420,13 +1456,17 @@ static const GtkActionEntry action_entries_file[] = {
 };
 
 static const GtkActionEntry action_entries_photo[] = {
-  {"TakePhoto", NULL, N_("_Take a photo"), "space", NULL, G_CALLBACK (cheese_window_action_button_clicked_cb)},
+  {"TakePhoto", NULL, N_("_Take a Photo"), "space", NULL, G_CALLBACK (cheese_window_action_button_clicked_cb)},
 };
 
 static const GtkToggleActionEntry action_entries_video[] = {
   {"TakeVideo", NULL, N_("_Recording"), "space", NULL, G_CALLBACK (cheese_window_action_button_clicked_cb), FALSE},
 };
 
+static const GtkActionEntry action_entries_burst[] = {
+  {"TakeBurst", NULL, N_("_Take multiple Photos"), "space", NULL, G_CALLBACK (cheese_window_action_button_clicked_cb)},
+};
+
 static const GtkActionEntry action_entries_account_photo[] = {
   {"SetAsAccountPhoto", NULL, N_("_Set As Account Photo"), NULL, NULL, G_CALLBACK (cheese_window_cmd_set_about_me_photo)},
 };
@@ -1463,6 +1503,20 @@ cheese_window_activate_radio_action (GtkAction *action, GtkRadioAction *current,
     gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo_fullscreen), TRUE);
     gtk_action_group_set_sensitive (cheese_window->actions_photo, TRUE);
     gtk_action_group_set_sensitive (cheese_window->actions_video, FALSE);
+    gtk_action_group_set_sensitive (cheese_window->actions_burst, FALSE);
+  }
+  else if (strcmp (gtk_action_get_name (GTK_ACTION (current)), "Burst") == 0)
+  {
+    cheese_window->webcam_mode = WEBCAM_MODE_BURST;
+
+    str = g_strconcat ("<b>", _("_Take multiple Photos"), "</b>", NULL);
+    gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), g_strdup (str));
+    gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
+    gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo_fullscreen), g_strdup (str));
+    gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo_fullscreen), TRUE);
+    gtk_action_group_set_sensitive (cheese_window->actions_burst, TRUE);
+    gtk_action_group_set_sensitive (cheese_window->actions_video, FALSE);
+    gtk_action_group_set_sensitive (cheese_window->actions_photo, FALSE);
   }
   else
   {
@@ -1475,6 +1529,7 @@ cheese_window_activate_radio_action (GtkAction *action, GtkRadioAction *current,
     gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo_fullscreen), TRUE);
     gtk_action_group_set_sensitive (cheese_window->actions_photo, FALSE);
     gtk_action_group_set_sensitive (cheese_window->actions_video, TRUE);
+    gtk_action_group_set_sensitive (cheese_window->actions_burst, FALSE);
   }
   g_free (str);
 }
@@ -1527,22 +1582,22 @@ cheese_window_radio_action_group_new (CheeseWindow *cheese_window, char *name,
 }
 
 static void
-cheese_window_set_message_area (CheeseWindow *cheese_window,
-                                GtkWidget    *message_area)
+cheese_window_set_info_bar (CheeseWindow *cheese_window,
+                            GtkWidget    *info_bar)
 {
-  if (cheese_window->message_area == message_area)
+  if (cheese_window->info_bar == info_bar)
     return;
 
-  if (cheese_window->message_area != NULL)
-    gtk_widget_destroy (cheese_window->message_area);
+  if (cheese_window->info_bar != NULL)
+    gtk_widget_destroy (cheese_window->info_bar);
 
-  cheese_window->message_area = message_area;
+  cheese_window->info_bar = info_bar;
 
-  if (message_area == NULL)
+  if (info_bar == NULL)
     return;
 
-  gtk_container_add (GTK_CONTAINER (cheese_window->message_area_frame), cheese_window->message_area);
-  gtk_widget_show (GTK_WIDGET (cheese_window->message_area));
+  gtk_container_add (GTK_CONTAINER (cheese_window->info_bar_frame), cheese_window->info_bar);
+  gtk_widget_show (GTK_WIDGET (cheese_window->info_bar));
 }
 
 static void
@@ -1561,7 +1616,7 @@ cheese_window_create_window (CheeseWindow *cheese_window)
 #endif
 
 
-  cheese_window->message_area = NULL;
+  cheese_window->info_bar = NULL;
 
   builder = gtk_builder_new ();
   gtk_builder_add_from_file (builder, PACKAGE_DATADIR "/cheese.ui", &error);
@@ -1576,6 +1631,7 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->button_effects              = GTK_WIDGET (gtk_builder_get_object (builder, "button_effects"));
   cheese_window->button_photo                = GTK_WIDGET (gtk_builder_get_object (builder, "button_photo"));
   cheese_window->button_video                = GTK_WIDGET (gtk_builder_get_object (builder, "button_video"));
+  cheese_window->button_burst                = GTK_WIDGET (gtk_builder_get_object (builder, "button_burst"));
   cheese_window->image_take_photo            = GTK_WIDGET (gtk_builder_get_object (builder, "image_take_photo"));
   cheese_window->label_effects               = GTK_WIDGET (gtk_builder_get_object (builder, "label_effects"));
   cheese_window->label_photo                 = GTK_WIDGET (gtk_builder_get_object (builder, "label_photo"));
@@ -1592,12 +1648,13 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->countdown_frame             = GTK_WIDGET (gtk_builder_get_object (builder, "countdown_frame"));
   cheese_window->effect_frame                = GTK_WIDGET (gtk_builder_get_object (builder, "effect_frame"));
   cheese_window->effect_alignment            = GTK_WIDGET (gtk_builder_get_object (builder, "effect_alignment"));
-  cheese_window->message_area_frame          = GTK_WIDGET (gtk_builder_get_object (builder, "message_area_frame"));
+  cheese_window->info_bar_frame              = GTK_WIDGET (gtk_builder_get_object (builder, "info_bar_frame"));
   cheese_window->fullscreen_popup            = GTK_WIDGET (gtk_builder_get_object (builder, "fullscreen_popup"));
   cheese_window->fullscreen_bar              = GTK_WIDGET (gtk_builder_get_object (builder, "fullscreen_notebook_bar"));
   cheese_window->button_effects_fullscreen   = GTK_WIDGET (gtk_builder_get_object (builder, "button_effects_fullscreen"));
   cheese_window->button_photo_fullscreen     = GTK_WIDGET (gtk_builder_get_object (builder, "button_photo_fullscreen"));
   cheese_window->button_video_fullscreen     = GTK_WIDGET (gtk_builder_get_object (builder, "button_video_fullscreen"));
+  cheese_window->button_burst_fullscreen     = GTK_WIDGET (gtk_builder_get_object (builder, "button_burst_fullscreen"));
   cheese_window->take_picture_fullscreen     = GTK_WIDGET (gtk_builder_get_object (builder, "take_picture_fullscreen"));
   cheese_window->label_take_photo_fullscreen =
     GTK_WIDGET (gtk_builder_get_object (builder, "label_take_photo_fullscreen"));
@@ -1662,6 +1719,9 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->thumb_nav  = eog_thumb_nav_new (cheese_window->thumb_view, TRUE);
   gtk_container_add (GTK_CONTAINER (cheese_window->thumb_scrollwindow), cheese_window->thumb_nav);
 
+  /* show the scroll window to get it included in the size requisition done later */
+  gtk_widget_show_all (cheese_window->thumb_scrollwindow);
+
   char *gconf_effects;
   g_object_get (cheese_window->gconf, "gconf_prop_selected_effects", &gconf_effects, NULL);
   cheese_window->effect_chooser = cheese_effect_chooser_new (gconf_effects);
@@ -1680,6 +1740,19 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->countdown_fullscreen = cheese_countdown_new ();
   gtk_container_add (GTK_CONTAINER (cheese_window->countdown_frame_fullscreen), cheese_window->countdown_fullscreen);
 
+  gtk_widget_realize (cheese_window->screen);
+  GdkWindow *win = gtk_widget_get_window (cheese_window->screen);
+  if (!gdk_window_ensure_native (win))
+  {
+    /* FIXME: this breaks offscreen stuff, we should really find
+     * another way to embed video that doesn't require an XID */
+
+    /* abort: no native window, no xoverlay, no cheese. */
+    g_error ("Could not create a native X11 window for the drawing area");
+  }
+  gdk_window_set_back_pixmap (gtk_widget_get_window (cheese_window->screen), NULL, FALSE);
+  gtk_widget_set_app_paintable (cheese_window->screen, TRUE);
+  gtk_widget_set_double_buffered (cheese_window->screen, FALSE);
   gtk_widget_add_events (cheese_window->screen, GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
 
   cheese_window->ui_manager = gtk_ui_manager_new ();
@@ -1723,6 +1796,11 @@ cheese_window_create_window (CheeseWindow *cheese_window)
                                                                         action_entries_video,
                                                                         G_N_ELEMENTS (action_entries_video));
   gtk_action_group_set_sensitive (cheese_window->actions_video, FALSE);
+  cheese_window->actions_burst = cheese_window_action_group_new (cheese_window,
+                                                                 "ActionsBurst",
+                                                                 action_entries_burst,
+                                                                 G_N_ELEMENTS (action_entries_burst));
+  gtk_action_group_set_sensitive (cheese_window->actions_burst, FALSE);
   cheese_window->actions_account_photo = cheese_window_action_group_new (cheese_window,
                                                                          "ActionsAccountPhoto",
                                                                          action_entries_account_photo,
@@ -1834,6 +1912,10 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   gtk_activatable_set_related_action (GTK_ACTIVATABLE (cheese_window->button_video), action);
   gtk_activatable_set_related_action (GTK_ACTIVATABLE (cheese_window->button_video_fullscreen), action);
 
+  action = gtk_ui_manager_get_action (cheese_window->ui_manager, "/MainMenu/Cheese/Burst");
+  gtk_activatable_set_related_action (GTK_ACTIVATABLE (cheese_window->button_burst), action);
+  gtk_activatable_set_related_action (GTK_ACTIVATABLE (cheese_window->button_burst_fullscreen), action);
+
 
   /* Default handlers for closing the application */
   g_signal_connect (cheese_window->window, "destroy",
@@ -1841,6 +1923,11 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   g_signal_connect (cheese_window->window, "delete_event",
                     G_CALLBACK (cheese_window_delete_event_cb), NULL);
 
+  /* Listen for key presses */
+  gtk_widget_add_events (cheese_window->window, GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
+  g_signal_connect (cheese_window->window, "key_press_event",
+                    G_CALLBACK (cheese_window_key_press_event_cb), cheese_window);
+
   g_signal_connect (cheese_window->take_picture, "clicked",
                     G_CALLBACK (cheese_window_action_button_clicked_cb), cheese_window);
   g_signal_connect (cheese_window->take_picture_fullscreen, "clicked",
@@ -1861,7 +1948,7 @@ setup_camera (CheeseWindow *cheese_window)
   gdouble    contrast;
   gdouble    saturation;
   gdouble    hue;
-  GtkWidget *message_area;
+  GtkWidget *info_bar;
 
   GError *error;
 
@@ -1935,18 +2022,19 @@ setup_camera (CheeseWindow *cheese_window)
   ephy_spinner_stop (EPHY_SPINNER (cheese_window->throbber));
   if (cheese_webcam_get_num_webcam_devices (cheese_window->webcam) == 0)
   {
-    message_area = cheese_no_camera_message_area ();
+    info_bar = cheese_no_camera_info_bar_new ();
 
-    g_signal_connect (message_area,
+    g_signal_connect (info_bar,
                       "response",
-                      G_CALLBACK (cheese_window_no_camera_message_area_response),
+                      G_CALLBACK (cheese_window_no_camera_info_bar_response),
                       cheese_window);
 
-    cheese_window_set_message_area (cheese_window, message_area);
+    cheese_window_set_info_bar (cheese_window, info_bar);
   }
   gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture), TRUE);
   gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture_fullscreen), TRUE);
   gtk_action_group_set_sensitive (cheese_window->actions_effects, TRUE);
+
   gdk_threads_leave ();
 }
 
@@ -1974,7 +2062,6 @@ cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
   cheese_window_create_window (cheese_window);
   gtk_action_group_set_sensitive (cheese_window->actions_effects, FALSE);
 
-  gtk_widget_show_all (cheese_window->window);
   ephy_spinner_start (EPHY_SPINNER (cheese_window->throbber));
 
   gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->notebook), 2);
@@ -1982,6 +2069,17 @@ cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
   cheese_window->webcam_mode = WEBCAM_MODE_PHOTO;
   cheese_window->recording   = FALSE;
 
+  /* handy trick to set default size of the drawing area while not
+   * limiting its minimum size, thanks Owen! */
+  GtkRequisition req;
+  gtk_widget_set_size_request (cheese_window->screen,
+                               DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT);
+  gtk_widget_size_request (cheese_window->window, &req);
+  gtk_window_set_default_size (GTK_WINDOW (cheese_window->window), req.width, req.height);
+  gtk_widget_set_size_request (cheese_window->screen, -1, -1);
+
+  gtk_widget_show_all (cheese_window->window);
+
   /* Run cam setup in its own thread */
   GError *error = NULL;
   if (!g_thread_create ((GThreadFunc) setup_camera, cheese_window, FALSE, &error))
@@ -1990,8 +2088,4 @@ cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
     g_error_free (error);
     return;
   }
-
-  /* Make URLs and email clickable in about dialog */
-  gtk_about_dialog_set_url_hook (cheese_about_dialog_handle_url, NULL, NULL);
-  gtk_about_dialog_set_email_hook (cheese_about_dialog_handle_email, NULL, NULL);
 }
